@@ -1,27 +1,27 @@
-# Copyright 2012 the authors
-# This file is part of mdtraj
+##############################################################################
+# MDTraj: A Python Library for Loading, Saving, and Manipulating
+#         Molecular Dynamics Trajectories.
+# Copyright 2012-2013 Stanford University and the Authors
 #
-# mdtraj is free software: you can redistribute it and/or modify it under the
-# terms of the GNU General Public License as published by the Free Software
-# Foundation, either version 3 of the License, or (at your option) any later
-# version.
-#
-# mdtraj is distributed in the hope that it will be useful, but WITHOUT ANY
-# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License along with
-# mdtraj. If not, see http://www.gnu.org/licenses/.
-#
-# topology.py: Used for storing topological information about a system.
-#
-# This is part of the OpenMM molecular simulation toolkit originating from
-# Simbios, the NIH National Center for Physics-Based Simulation of
-# Biological Structures at Stanford, funded under the NIH Roadmap for
-# Medical Research, grant U54 GM072970. See https://simtk.org.
-#
-# Portions copyright (c) 2012 Stanford University and the Authors.
 # Authors: Peter Eastman, Robert McGibbon
+# Contributors: Kyle A. Beauchamp
+#
+# MDTraj is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as
+# published by the Free Software Foundation, either version 2.1
+# of the License, or (at your option) any later version.
+#
+# This library is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public
+# License along with MDTraj. If not, see <http://www.gnu.org/licenses/>.
+#
+# Portions of this code originate from the OpenMM molecular simulation
+# toolkit, copyright (c) 2012 Stanford University and Peter Eastman. Those
+# portions are distributed under the following terms:
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -40,14 +40,17 @@
 # DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 # OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
 # USE OR OTHER DEALINGS IN THE SOFTWARE.
+##############################################################################
+
 
 ##############################################################################
 # Imports
 ##############################################################################
 
-import cPickle as pickle
+from __future__ import print_function, division
 import os
 import numpy as np
+import itertools
 import xml.etree.ElementTree as etree
 
 from mdtraj.utils import ilen, import_
@@ -80,7 +83,8 @@ def _topology_from_subset(topology, atom_indices):
     for chain in topology._chains:
         newChain = newTopology.add_chain()
         for residue in chain._residues:
-            newResidue = newTopology.add_residue(residue.name, newChain)
+            resSeq = getattr(residue, 'resSeq', None) or residue.index
+            newResidue = newTopology.add_residue(residue.name, newChain, resSeq)
             for atom in residue._atoms:
                 if atom.index in atom_indices:
                     newAtom = newTopology.add_atom(atom.name, atom.element, newResidue)
@@ -140,10 +144,10 @@ class Topology(object):
     Examples
     --------
     >>> topology = md.load('example.pdb').topology            # doctest: +SKIP
-    >>> print topology                                        # doctest: +SKIP
+    >>> print(topology)                                       # doctest: +SKIP
     <mdtraj.Topology with 1 chains, 3 residues, 22 atoms, 21 bonds at 0x105a98e90>
     >>> table, bonds = topology.to_dataframe()                # doctest: +SKIP
-    >>> print table.head()                                    # doctest: +SKIP
+    >>> print(table.head())                                   # doctest: +SKIP
        serial name element  resSeq resName  chainID
     0       0   H1       H       0     CYS        0
     1       1  CH3       C       0     CYS        0
@@ -152,7 +156,7 @@ class Topology(object):
     4       4    C       C       0     CYS        0
     >>> # rename residue "CYS" to "CYSS"
     >>> table[table['residue'] == 'CYS']['residue'] = 'CYSS'  # doctest: +SKIP
-    >>> print table.head()                                    # doctest: +SKIP
+    >>> print(table.head())                                   # doctest: +SKIP
        serial name element  resSeq resName   chainID
     0       0   H1       H       0     CYSS        0
     1       1  CH3       C       0     CYSS        0
@@ -194,7 +198,7 @@ class Topology(object):
         for chain in self.chains:
             c = out.add_chain()
             for residue in chain.residues:
-                r = out.add_residue(residue.name, c)
+                r = out.add_residue(residue.name, c, residue.resSeq)
                 for atom in residue.atoms:
                     out.add_atom(atom.name, atom.element, r)
 
@@ -231,7 +235,7 @@ class Topology(object):
         for chain in other.chains:
             c = out.add_chain()
             for residue in chain.residues:
-                r = out.add_residue(residue.name, c)
+                r = out.add_residue(residue.name, c, residue.resSeq)
                 for atom in residue.atoms:
                     a = out.add_atom(atom.name, atom.element, r)
                     atom_mapping[atom] = a
@@ -314,8 +318,12 @@ class Topology(object):
         pd = import_('pandas')
         data = []
         for atom in self.atoms:
-            data.append((atom.index, atom.name, atom.element.symbol,
-                         atom.residue.index, atom.residue.name,
+            if atom.element is None:
+                element_symbol = ""
+            else:
+                element_symbol = atom.element.symbol
+            data.append((atom.index, atom.name, element_symbol,
+                         atom.residue.resSeq, atom.residue.name,
                          atom.residue.chain.index))
 
         atoms = pd.DataFrame(data, columns=["serial", "name", "element",
@@ -375,10 +383,14 @@ class Topology(object):
                 residue_name = np.array(rnames)[0]
                 if not np.all(rnames == residue_name):
                     raise ValueError('All of the atoms with residue index %d do not share the same residue name' % ri)
-                r = out.add_residue(residue_name, c)
+                r = out.add_residue(residue_name, c, ri)
 
                 for ai, atom in residue_atoms.iterrows():
-                    a = Atom(atom['name'], pdb.element.get_by_symbol(atom['element']), ai, r)
+                    if atom['element'] == "":
+                        element = None
+                    else:
+                        element = pdb.element.get_by_symbol(atom['element'])
+                    a = Atom(atom['name'], element, ai, r)
                     out._atoms[ai] = a
                     r._atoms.append(a)
 
@@ -386,6 +398,7 @@ class Topology(object):
             for ai1, ai2 in bonds:
                 out.add_bond(out.atom(ai1), out.atom(ai2))
 
+        out._numAtoms = out.n_atoms
         return out
 
     def __eq__(self, other):
@@ -416,7 +429,7 @@ class Topology(object):
                 return False
 
             for r1, r2 in zip(c1.residues, c2.residues):
-                if (r1.index != r1.index) or (r1.name != r2.name):
+                if (r1.index != r1.index) or (r1.name != r2.name): # or (r1.resSeq != r2.resSeq):
                     return False
                 if len(r1._atoms) != len(r2._atoms):
                     return False
@@ -424,9 +437,10 @@ class Topology(object):
                 for a1, a2 in zip(r1.atoms, r2.atoms):
                     if (a1.index != a2.index)  or (a1.name != a2.name):
                         return False
-                    for attr in ['atomic_number', 'name', 'symbol']:
-                        if getattr(a1.element, attr) != getattr(a2.element, attr):
-                            return False
+                    if a1.element is not None and a2.element is not None:
+                        for attr in ['atomic_number', 'name', 'symbol']:
+                            if getattr(a1.element, attr) != getattr(a2.element, attr):
+                                return False
 
         if len(self._bonds) != len(other._bonds):
             return False
@@ -448,7 +462,7 @@ class Topology(object):
         self._chains.append(chain)
         return chain
 
-    def add_residue(self, name, chain):
+    def add_residue(self, name, chain, resSeq=None):
         """Create a new Residue and add it to the Topology.
 
         Parameters
@@ -457,13 +471,20 @@ class Topology(object):
             The name of the residue to add
         chain : mdtraj.topology.Chain
             The Chain to add it to
+        resSeq : int, optional
+            Residue sequence number, such as from a PDB record. These sequence
+            numbers are arbitrary, and do not necessarily start at 0 (or 1).
+            If not supplied, the resSeq attribute will be set to the
+            residue's sequential (0 based) index.
 
         Returns
         -------
         residue : mdtraj.topology.Residue
             The newly created Residue
         """
-        residue = Residue(name, self._numResidues, chain)
+        if resSeq is None:
+            resSeq = self._numResidues
+        residue = Residue(name, self._numResidues, chain, resSeq)
         self._residues.append(residue)
         self._numResidues += 1
         chain._residues.append(residue)
@@ -724,6 +745,20 @@ class Chain(object):
         """
         return iter(self._residues)
 
+    def residue(self, index):
+        """Get a specific residue in this Chain
+
+        Returns
+        -------
+        residue : Residue
+        """
+        return self._residue[index]
+
+    @property
+    def n_residues(self):
+        "Get the number of residues in this Chain"
+        return len(self._residues)
+
     @property
     def atoms(self):
         """Iterator over all Atoms in the Chain.
@@ -737,22 +772,42 @@ class Chain(object):
             for atom in residue._atoms:
                 yield atom
 
+    def atom(self, index):
+        """Get a specific atom in this Chain
+
+        Returns
+        -------
+        atom : Atom
+        """
+        # this could be made faster by caching the list
+        # of atoms internally if necessary
+        return next(itertools.islice(self.atoms, index, index+1))
+
+    @property
+    def n_atoms(self):
+        """Get the number of atoms in this Chain"""
+        return sum(r.n_atoms for r in self._residues)
+
+
 class Residue(object):
     """A Residue object represents a residue within a Topology.
 
     Attributes
     ----------
-    atoms : genetator
+    name : str
+        The name of the Residue
+    index : int
+        The index of the Residue within its Topology
+    chain : int
+        The residue sequence number
     """
-    def __init__(self, name, index, chain):
+    def __init__(self, name, index, chain, resSeq):
         """Construct a new Residue.  You should call add_residue()
         on the Topology instead of calling this directly."""
-        ## The name of the Residue
         self.name = name
-        ## The index of the Residue within its Topology
         self.index = index
-        ## The Chain this Residue belongs to
         self.chain = chain
+        self.resSeq = resSeq
         self._atoms = []
 
     @property
@@ -766,6 +821,21 @@ class Residue(object):
         """
         return iter(self._atoms)
 
+    def atom(self, index):
+        """Get a specific atom in this Residue.
+
+        Returns
+        -------
+        atom : Atom
+        """
+        return self._atoms[index]
+
+    @property
+    def n_atoms(self):
+        """Get the number of atoms in this Residue"""
+        return len(self._atoms)
+
+
 class Atom(object):
     """An Atom object represents a residue within a Topology.
 
